<?php
// Developer mode active? Comment out if no dev!
define('DEVELOPER', true);
//xdebug_start_trace();
/**
 * The main class with the entry point to the whole application. This class
 * "emulates" Java(tm)'s entry point call. Additionally it covers local
 * variables from outside access to prevent possible attacks on uninitialized
 * local variables.
 *
 * But good little boys and girls would always initialize their variables... ;-)
 *
 * @author		Roland Haeder <webmaster@ship-simu.org>
 * @version		0.0.0
 * @copyright	Copyright (c) 2007, 2008 Roland Haeder, this is free software
 * @license		GNU GPL 3.0 or any newer version
 * @link		http://www.ship-simu.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
class ApplicationEntryPoint {
	/**
	 * The instances we want to remove after all is done
	 *
	 * @return	void
	 */
	private static $instances = array (
		'cfg',    // The configuration system
		'loader', // The class loader system
		'debug',  // Debug output
		'db',     // Database layer
		'io',     // Base I/O system (local file [or network])
		'engine', // Template engine ( for ApplicationEntryPoint::app_die() )
		'lang',   // Language sub-system
		'app',    // The ApplicationHelper instance
	);

	/**
	 * The application's emergency exit
	 *
	 * @param	$message		The optional message we shall output on exit
	 * @param	$code			Error code from exception
	 * @param	$extraData		Extra information from exceptions
	 * @param	$silentMode		Wether not silent mode is turned on
	 * @return	void
	 */
	public static function app_die ($message = "", $code = false, $extraData = "", $silentMode = false) {
		// Is this method already called?
		if (defined('EMERGENCY_EXIT_CALLED')) {
			// Then output the text directly
			die($message);
		} // END - if

		// This method shall not be called twice
		define('EMERGENCY_EXIT_CALLED', true);

		// Is a message set?
		if (empty($message)) {
			// No message provided
			$message = "No message provided!";
		} // END - if

		// Get config instance
		$configInstance = FrameworkConfiguration::getInstance();

		// Do we have debug installation?
		if (($configInstance->readConfig('product_install_mode') == "productive") || ($silentMode === true)) {
			// Abort here
			die();
		} // END - if

		// Get some instances
		$tpl = FrameworkConfiguration::getInstance()->readConfig('template_class');
		$lang = LanguageSystem::getInstance();
		$io = FileIoHandler::getInstance();

		// Get response instance
		$responseInstance = ApplicationHelper::getInstance()->getResponseInstance();

		// Is the template engine loaded?
		if ((class_exists($tpl)) && (is_object($lang)) && (is_object($io))) {
			// Use the template engine for putting out (nicer look) the message
			try {
				// Get the template instance from our object factory
				$templateInstance = ObjectFactory::createObjectByName($tpl, array(FrameworkConfiguration::getInstance()->readConfig('tpl_base_path'), $lang, $io));
			} catch (FrameworkException $e) {
				die(sprintf("[Main:] Could not initialize template engine for reason: <span class=\"exception_reason\">%s</span>",
					$e->getMessage()
				));
			}

			// Get and prepare backtrace for output
			$backtraceArray = debug_backtrace();
			$backtrace = "";
			foreach ($backtraceArray as $key => $trace) {
				if (!isset($trace['file'])) $trace['file'] = __FILE__;
				if (!isset($trace['line'])) $trace['line'] = __LINE__;
				if (!isset($trace['args'])) $trace['args'] = array();
				$backtrace .= "<span class=\"backtrace_file\">".basename($trace['file'])."</span>:".$trace['line'].", <span class=\"backtrace_function\">".$trace['function']."(".count($trace['args']).")</span><br />";
			} // END - foreach

			// Init application instance
			$appInstance = null;

			// Is the class there?
			if (class_exists('ApplicationHelper')) {
				// Get application instance
				$appInstance = ApplicationHelper::getInstance();

				// Assign application data
				$templateInstance->assignApplicationData($appInstance);
			} // END - if

			// Assign variables
			$templateInstance->assignVariable('message', $message);
			$templateInstance->assignVariable('code', $code);
			$templateInstance->assignVariable('extra', $extraData);
			$templateInstance->assignVariable('backtrace', $backtrace);
			$templateInstance->assignVariable('total_includes', ClassLoader::getInstance()->getTotal());
			$templateInstance->assignVariable('total_objects', ObjectFactory::getTotal());
			$templateInstance->assignVariable('title', $lang->getMessage('emergency_exit_title'));

			// Load the template
			$templateInstance->loadCodeTemplate('emergency_exit');

			// Compile the template
			$templateInstance->compileTemplate();

			// Compile all variables
			$templateInstance->compileVariables();

			// Transfer data to response
			$templateInstance->transferToResponse($responseInstance);

			// Flush the response
			$responseInstance->flushBuffer();

			// Good bye...
			exit();
		} else {
			// Output message and die
			die(sprintf("[Main:] Emergency exit reached: <span class=\"emergency_span\">%s</span>",
				$message
			));
		}
	}

	/**
	 * The application's main entry point. This class isolates some local
	 * variables which shall not become visible to outside because of security
	 * concerns. We are doing this here to "emulate" the well-known entry
	 * point in Java(tm).
	 *
	 * @return	void
	 */
	public static function main () {
		// Some non-global common arrays we need...
		global $_SERVER;

		// Load config file
		require(dirname(__FILE__) . '/inc/config.php');

		// Load all include files
		require($cfg->readConfig('base_path') . 'inc/includes.php');

		// Load all framework classes
		require($cfg->readConfig('base_path') . 'inc/classes.php');

		// Include the application selector
		require($cfg->readConfig('base_path') . 'inc/selector.php');
	} // END - main()

} // END - class

// Do not remove the following line:
ApplicationEntryPoint::main();

// [EOF]
?>
